SystemOrganization addCategory: #'DuinoBot-Core'!
SystemOrganization addCategory: #'DuinoBot-Core-Firmware'!
SystemOrganization addCategory: #'DuinoBot-Core-Hid'!

ArduinoDirectMode subclass: #DuinoBotDirectMode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DuinoBot-Core-Firmware'!

!DuinoBotDirectMode class methodsFor: 'as yet unclassified' stamp: 'Richo 1/7/2013 15:47'!
rqMotorDC
	^ 9! !

!DuinoBotDirectMode class methodsFor: 'as yet unclassified' stamp: 'Richo 1/16/2013 17:26'!
rqPlayTone
	^ 10! !

!DuinoBotDirectMode methodsFor: 'connection' stamp: 'Richo 12/4/2012 16:55'!
connectOnHIDPort: aString
	port := HIDPort new.
	port := port 	openPort: aString! !

!DuinoBotDirectMode methodsFor: 'connection' stamp: 'Richo 12/4/2012 16:56'!
connectOnPort: aString
	self isConnected ifTrue: [^ self].
	('HID' match: aString)
		ifTrue: [self connectOnHIDPort: aString]
		ifFalse: [self connectOnSerialPort: aString].
	self isConnected
		ifTrue: [self controlFirmwareInstalled; startSteppingProcess]! !

!DuinoBotDirectMode methodsFor: 'connection' stamp: 'Richo 12/4/2012 16:56'!
connectOnSerialPort: aString
	"If the port cant't be open it stores nil, this way we can know if it is
	connected by asking if port is nil.
	In windows we can't open a port using the port name, so I convert the
	string to an integer"
	port := SerialPort new baudRate: 57600.
	port := port
				openPort: (('COM*' match: aString)
						ifTrue: [aString asInteger]
						ifFalse: [aString])! !

!DuinoBotDirectMode methodsFor: 'as yet unclassified' stamp: 'Richo 4/18/2012 17:00'!
controlFirmwareInstalled
	| time timeout |
	time := Time millisecondClockValue.
	timeout := 5000.
	[100 milliSeconds asDelay wait.
	('*HOLA, SOY DUINOBOT*' match: port readString)
		ifTrue: [
			port nextPutAll: (ByteArray with: 255).
			[port readString = ''] whileFalse.
			^ self]]
		doWhileTrue: [Time millisecondClockValue - time < timeout].
	^ FirmwareNotInstalled signal! !

!DuinoBotDirectMode methodsFor: 'as yet unclassified' stamp: 'Richo 8/31/2011 17:33'!
motorDC: motorIndex speed: pinValue
	| speed |
	speed := (1.275 * pinValue + 127.5) rounded.
	self send: {self asCommand: self class rqMotorDC.
						self asArgument: motorIndex.
						self asArgument: (speed bitAnd: 2r01111111).
						self asArgument: (speed bitAnd: 2r10000000) >> 7} asByteArray! !

!DuinoBotDirectMode methodsFor: 'as yet unclassified' stamp: 'Richo 1/16/2013 17:38'!
playTone: frequency duration: duration pin: pin
	| freq dur |
	freq := frequency rounded.
	dur := duration rounded.
	self send: {self asCommand: self class rqPlayTone.
				"pin"
				self asArgument: pin.
				"frequency"
				self asArgument: (freq bitAnd: 2r1111111).
				self asArgument: ((freq bitAnd: 2r11111110000000) >> 7).
				self asArgument: ((freq bitAnd: 2r111111100000000000000) >> 14) .
				"duration"
				self asArgument: (dur bitAnd: 2r1111111).
				self asArgument: ((dur bitAnd: 2r11111110000000) >> 7)} asByteArray! !

Arduino subclass: #DuinoBot
	instanceVariableNames: 'motorPins'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DuinoBot-Core'!

!DuinoBot class methodsFor: 'as yet unclassified' stamp: 'Richo 5/15/2012 15:10'!
defaultProtocol
	^ DuinoBotDirectMode! !

!DuinoBot methodsFor: 'initialize-release' stamp: 'Richo 3/6/2012 09:57'!
initialize
	super initialize.
	motorPins := Dictionary newFrom: ((0 to: 1)
				collect: [:i | i -> (DuinoBotMotorPin on: self number: i)])! !

!DuinoBot methodsFor: 'motors' stamp: 'Richo 8/31/2011 15:20'!
motorPin: index
	^ motorPins at: index ifAbsent: [self inaccessiblePinError]! !

!DuinoBot methodsFor: 'accessing' stamp: 'Richo 10/24/2012 15:44'!
motorPins
	^ motorPins values
! !

!DuinoBot methodsFor: 'accessing' stamp: 'Richo 10/24/2012 15:56'!
pins
	^ super pins, self motorPins! !

!DuinoBot methodsFor: 'initialize-release' stamp: 'Richo 4/18/2012 13:34'!
reconfigurePins
	"Overriden to avoid DuinoBot issue on Linux"
	digitalPins values , analogPins values
		do: [:each | 
			100 milliSeconds asDelay wait.
			each refresh]! !

ArduinoType subclass: #DuinoBotType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DuinoBot-Core'!

!DuinoBotType class methodsFor: 'accessing types' stamp: 'Richo 11/23/2012 12:34'!
default
	^ self at: #'DuinoBot.v1.x.HID'! !

!DuinoBotType class methodsFor: 'accessing types' stamp: 'Richo 11/23/2012 12:36'!
hidTypes
	^ {self at: #'DuinoBot.v1.x.HID'}! !

!DuinoBotType class methodsFor: 'class initialization' stamp: 'Richo 11/23/2012 13:38'!
initialize
	"
	self initialize.
	"
	types := Dictionary new.

	types
		at: #DuinoBotCDC
		put: (DuinoBotType
			friendlyNames: #('duinobot 1.x' )
			fCpu: 16000000
			mcu: 'atmega32u4'
			uploadRate: 115200
			firmataRate: 57600
			avrdudeProgrammer: 'avr109').

	types
		at: #'DuinoBot.v1.x.HID'
		put: (DuinoBotType
			friendlyNames: #('duinobot 1.x hid' )
			fCpu: 16000000
			mcu: 'atmega32u4'
			uploadRate: 115200
			firmataRate: 57600
			avrdudeProgrammer: 'avr109')! !

!DuinoBotType methodsFor: 'testing' stamp: 'Richo 11/23/2012 12:36'!
isHID
	^ self class hidTypes includes: self! !

Object subclass: #HID
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DuinoBot-Core-Hid'!

!HID class methodsFor: 'utils' stamp: 'Richo 12/4/2012 14:04'!
close: ptr
	self apiClose: ptr.
	self apiExit = -1
		ifTrue: [^ self error: 'HID finalization error' translated]
	! !

!HID class methodsFor: 'error' stamp: 'Richo 12/5/2012 16:47'!
error: aString 
	"Throw a HIDError exception."
	^ HIDError new signal: aString! !

!HID class methodsFor: 'accessing' stamp: 'Richo 12/7/2012 10:48'!
forThisPlatform
	^ SmalltalkImage current platformName caseOf: {
		['unix'] -> [HIDUnix].
		['Win32'] -> [HIDWin32]
	} otherwise: [self error: 'Platform not supported' translated]! !

!HID class methodsFor: 'utils' stamp: 'Richo 12/4/2012 14:03'!
open: vid with: pid
	| ptr |
	self apiInit = -1
		ifTrue: [^ self error: 'HID initialization error' translated].
	ptr := self apiOpen: vid with: pid with: nil.
	ptr = 0
		ifTrue: [^ self error: 'HID opening device error' translated].
	^ ptr! !

!HID class methodsFor: 'utils' stamp: 'Richo 12/4/2012 17:56'!
read: ptr
	| bufSize buf bytesRead |
	bufSize := 8.	
	buf := ByteArray new: bufSize.
	bytesRead := self apiReadTimeout: ptr with: buf with: bufSize with: 0.
	^ bytesRead caseOf: {
		[bufSize] -> [buf].
		[0] -> [#[]].
		[-1] -> [self error: 'HID reading error' translated]
	} otherwise: [buf copyFrom: 1 to: bytesRead]! !

!HID class methodsFor: 'utils' stamp: 'Richo 12/4/2012 15:09'!
write: ptr with: data
	| bytesWritten |
	bytesWritten := self apiWrite: ptr with: data with: data size.
	bytesWritten = -1
		ifTrue: [^ self error: 'HID writing error' translated].
	^ bytesWritten! !

HID subclass: #HIDUnix
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DuinoBot-Core-Hid'!

!HIDUnix class methodsFor: 'api' stamp: 'Richo 12/7/2012 10:45'!
apiClose: device
"void HID_API_EXPORT HID_API_CALL hid_close(hid_device *device);"
	<apicall: void 'hid_close' (long) module: 'libhidapi'>
	^ self externalCallFailed! !

!HIDUnix class methodsFor: 'api' stamp: 'Richo 12/7/2012 10:37'!
apiExit
	<apicall: long 'hid_exit' () module: 'libhidapi'>
	^ self externalCallFailed! !

!HIDUnix class methodsFor: 'api' stamp: 'Richo 12/7/2012 10:36'!
apiInit	
	<apicall: long 'hid_init' () module: 'libhidapi'>
	^ self externalCallFailed! !

!HIDUnix class methodsFor: 'api' stamp: 'Richo 12/7/2012 10:45'!
apiOpen: vendorId with: productId with: serialNumber
"HID_API_EXPORT hid_device * HID_API_CALL hid_open(unsigned short vendor_id, unsigned short product_id, const wchar_t *serial_number);"
	<apicall: long 'hid_open' (ushort ushort short*) module: 'libhidapi'>
	^ self externalCallFailed! !

!HIDUnix class methodsFor: 'api' stamp: 'Richo 12/7/2012 10:45'!
apiRead: device with: buffer with: bufferSize
"int  HID_API_EXPORT HID_API_CALL hid_read(hid_device *device, unsigned char *data, size_t length);"
	<apicall: long 'hid_read' (long char* ulong) module: 'libhidapi'>
	^ self externalCallFailed! !

!HIDUnix class methodsFor: 'api' stamp: 'Richo 12/7/2012 10:45'!
apiReadTimeout: device with: buffer with: bufferSize with: timeout
"int HID_API_EXPORT HID_API_CALL hid_read_timeout(hid_device *dev, unsigned char *data, size_t length, int milliseconds);"
	<apicall: long 'hid_read_timeout' (long char* ulong long) module: 'libhidapi'>
	^ self externalCallFailed! !

!HIDUnix class methodsFor: 'api' stamp: 'Richo 12/7/2012 10:45'!
apiWrite: device with: buffer with: bufferSize
"int  HID_API_EXPORT HID_API_CALL hid_write(hid_device *device, const unsigned char *data, size_t length);"
	<apicall: long 'hid_write' (long char* ulong) module: 'libhidapi'>
	^ self externalCallFailed! !

HID subclass: #HIDWin32
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DuinoBot-Core-Hid'!

!HIDWin32 class methodsFor: 'api' stamp: 'Richo 12/7/2012 10:46'!
apiClose: device
"void HID_API_EXPORT HID_API_CALL hid_close(hid_device *device);"
	<apicall: void 'hid_close' (long) module: 'hidapi.dll'>
	^ self externalCallFailed! !

!HIDWin32 class methodsFor: 'api' stamp: 'Richo 12/7/2012 10:46'!
apiExit
	<apicall: long 'hid_exit' () module: 'hidapi.dll'>
	^ self externalCallFailed! !

!HIDWin32 class methodsFor: 'api' stamp: 'Richo 12/7/2012 10:46'!
apiInit	
	<apicall: long 'hid_init' () module: 'hidapi.dll'>
	^ self externalCallFailed! !

!HIDWin32 class methodsFor: 'api' stamp: 'Richo 12/7/2012 10:46'!
apiOpen: vendorId with: productId with: serialNumber
"HID_API_EXPORT hid_device * HID_API_CALL hid_open(unsigned short vendor_id, unsigned short product_id, const wchar_t *serial_number);"
	<apicall: long 'hid_open' (ushort ushort short*) module: 'hidapi.dll'>
	^ self externalCallFailed! !

!HIDWin32 class methodsFor: 'api' stamp: 'Richo 12/7/2012 10:46'!
apiRead: device with: buffer with: bufferSize
"int  HID_API_EXPORT HID_API_CALL hid_read(hid_device *device, unsigned char *data, size_t length);"
	<apicall: long 'hid_read' (long char* ulong) module: 'hidapi.dll'>
	^ self externalCallFailed! !

!HIDWin32 class methodsFor: 'api' stamp: 'Richo 12/7/2012 10:46'!
apiReadTimeout: device with: buffer with: bufferSize with: timeout
"int HID_API_EXPORT HID_API_CALL hid_read_timeout(hid_device *dev, unsigned char *data, size_t length, int milliseconds);"
	<apicall: long 'hid_read_timeout' (long char* ulong long) module: 'hidapi.dll'>
	^ self externalCallFailed! !

!HIDWin32 class methodsFor: 'api' stamp: 'Richo 12/7/2012 10:46'!
apiWrite: device with: buffer with: bufferSize
"int  HID_API_EXPORT HID_API_CALL hid_write(hid_device *device, const unsigned char *data, size_t length);"
	<apicall: long 'hid_write' (long char* ulong) module: 'hidapi.dll'>
	^ self externalCallFailed! !

Object subclass: #HIDPort
	instanceVariableNames: 'device'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DuinoBot-Core-Hid'!

!HIDPort class methodsFor: 'as yet unclassified' stamp: 'Richo 1/14/2013 17:28'!
initialize
	"
	self initialize
	"
	Smalltalk
		addToShutDownList: self
! !

!HIDPort class methodsFor: 'as yet unclassified' stamp: 'Richo 1/7/2013 13:40'!
shutDown
	self allInstancesDo: [:each | each close]! !

!HIDPort methodsFor: 'connection' stamp: 'Richo 12/7/2012 10:48'!
close
	[HID forThisPlatform close: device]
		ensure: [device := nil]! !

!HIDPort methodsFor: 'input/output' stamp: 'Richo 1/14/2013 17:30'!
nextPutAll: data
	| buf hid |
	device ifNil: [^ -1].
	hid := HID forThisPlatform.
	buf := #(0 16rB1 0 0 0 0 0 0 0) asByteArray.
	1 to: data size do: [:i |
		buf at: 3 put: (data at: i).
		[hid write: device with: buf]
			on: Error do: [^ -1]].
	^ data size
! !

!HIDPort methodsFor: 'connection' stamp: 'Richo 12/17/2012 16:44'!
openPort: portName
	('HID' match: portName) ifFalse: [^ self error: 'HID invalid port' translated].
	"First, try to open in console mode:"
	[device := HID forThisPlatform open: 16r3EB with: 16r204F]
		on: HIDError
		do: [| bootPacket |
			"Could not open. Try to open the bootloader"
			[device := HID forThisPlatform open: 16r3EB with: 16r2067]
				on: HIDError do: ["No DuinoBot found" ^ nil].
			"Bootloader found. Send reset command, so it starts the app"
			bootPacket := ByteArray new: 131 withAll: 0.
			bootPacket at: 2 put: 16rFF.
			bootPacket at: 3 put: 16rFF.
			1 second asDelay wait.
			HID forThisPlatform 
				apiWrite: device
				with: bootPacket
				with: bootPacket size.
			HID forThisPlatform close: device.
			1 second asDelay wait.
			[device := HID forThisPlatform open: 16r3EB with: 16r204F]
				on: HIDError do: ["No DuinoBot found" ^ nil]]! !

!HIDPort methodsFor: 'input/output' stamp: 'Richo 12/7/2012 10:49'!
readByteArray
	| result |
	device ifNil: [^ #() asByteArray].
	result := OrderedCollection new.
	1 to: 32 do: [:i || data |
		data := HID forThisPlatform read: device.
		(data size > 2
		and: [data first = 16rB1
		and: [data second ~= 0]])
			ifTrue: [result addAll: data allButFirst]].
	^ result asByteArray! !

!HIDPort methodsFor: 'input/output' stamp: 'Richo 12/4/2012 16:53'!
readString
	^ self readByteArray asString! !

AttachableDevice subclass: #RGMotor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DuinoBot-Core'!

!RGMotor methodsFor: 'as yet unclassified' stamp: 'Richo 9/7/2011 11:15'!
brake
	self controlAttaching; reconfigureIfNecessary.
	pins first value: 0! !

!RGMotor methodsFor: 'as yet unclassified' stamp: 'Richo 9/7/2011 11:15'!
configure
	pins first refresh! !

!RGMotor methodsFor: 'as yet unclassified' stamp: 'Richo 9/7/2011 11:15'!
deleteConfiguration
	pins first value: 0! !

!RGMotor methodsFor: 'accessing' stamp: 'Richo 10/24/2012 17:15'!
motorID
	^ pins first number! !

!RGMotor methodsFor: 'as yet unclassified' stamp: 'Richo 9/7/2011 11:10'!
speed
	self controlAttaching; reconfigureIfNecessary.
	^ pins first value! !

!RGMotor methodsFor: 'as yet unclassified' stamp: 'Richo 9/7/2011 11:11'!
speed: aNumber
	self controlAttaching; reconfigureIfNecessary.
	pins first value: aNumber! !

Error subclass: #HIDError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DuinoBot-Core-Hid'!

ArduinoPin subclass: #DuinoBotMotorPin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DuinoBot-Core'!

!DuinoBotMotorPin methodsFor: 'as yet unclassified' stamp: 'Richo 3/6/2012 09:56'!
refresh
	self protocol motorDC: number speed: value
! !

!DuinoBotMotorPin methodsFor: 'as yet unclassified' stamp: 'Richo 9/1/2011 14:19'!
value
	^ value! !

!DuinoBotMotorPin methodsFor: 'as yet unclassified' stamp: 'Richo 1/14/2013 16:52'!
value: speed
	value := speed min: 100 max: -100.
	self refresh! !
DuinoBotType initialize!
HIDPort initialize!
